
from pathlib import Path
import os
from subprocess import call

from .abs_module import Module

from termcolor import cprint, colored
from interutils import pr, cyan, is_package, choose, pause, ask

CHAR_FILE = Path('/usr/share/crunch/charset.lst')


def prompt_int(prompt: str) -> (int, None):
    try:
        return int(input(colored(prompt + ' >', 'yellow')))
    except Exception as e:
        pr('An exception caught!', 'X')
        return print(e)


def prompt_mask() -> (str, None):
    try:
        return input(colored('Mask special chars: \n\t@ will insert lower case characters\n\t, will insert upper case characters\n\t% will insert numbers\n\t^ will insert symbols\n\n> ', 'yellow'))
    except Exception as e:
        pr('An exception caught!', 'X')
        return print(e)


class Crunch(Module):
    def __init__(self, stargen):
        super().__init__(stargen, 'crun')

        # Identify existing crunches
        try:
            self.crunches = os.listdir(self.dest_dir)
        except FileNotFoundError:
            self.crunches = []

    def _gen(self, file_name: str, cmd: iter) -> (str, None):
        # Verify destination dir
        self.dest_dir.mkdir(exist_ok=True)
        ap: Path = self.dest_dir / file_name
        if ap.is_file():
            pr('Skipping, crunch already exist!')
            return file_name
        pr('Generating crunch: ' + cyan(file_name))
        cmd += ['-o', ap]
        call(cmd)
        if not ap.is_file():
            return
        return file_name

    def _crunch(self, crunch_method) -> None:
        try:
            # Crunch it
            cn = crunch_method()
            if not cn:
                return pr("Crunch was NOT generated!", 'X')
            self.crunches.append(cn)  # Save result
        except KeyboardInterrupt:
            print()
            pr('Interrupted!', '!')
            return

    def menu(self) -> tuple:
        return {
            # 'show': (self.show, 'Show crunches'),
            'crunch': (self.crunch, 'Generate a new wordlists via crunch\n\tOpt: "mask" -> Generate a wordlist based on a mask')
        }

    def show(self, args: tuple) -> None:
        pr(f'Destination directory: "{cyan(str(self.dest_dir))}"')
        if not self.crunches:
            return pr('No crunches downloaded yet!', '!')
        pr('Available crunches:')
        for p in self.crunches:
            cprint('  ' + p, 'yellow')
        pr(f'Crunches count: ' + cyan(len(self.crunches)))

    def crunch(self, args: tuple) -> None:
        if not is_package('crunch'):
            return pr('Package "crunch" not installed!', 'X')

        # Get args
        is_mask = args and args[0] == 'mask'

        # Mode switch: [Mask / Charset]
        if is_mask:
            mask = prompt_mask()
            if not mask:
                return pr('Invalid mask!', '!')

            file_name = f'mask_{ask("Enter save name:")}.dict'
            l = str(len(mask))
            return self._crunch(lambda: self._gen(file_name, ['crunch', l, l, '-t', mask]))

        # Get min and max seq len
        min_len = prompt_int('Enter minimum length')
        max_len = prompt_int('Enter maximum length')
        if not min_len or not max_len or \
                min_len < 1 or max_len < min_len:
            return pr('Invalid paramenters!', '!')

        # Ask for a charset to use
        with CHAR_FILE.open(encoding='utf-8') as char_file:
            sets = [n[:-1]
                    for n in char_file if '=' in n and 'sv' not in n]
        select = choose(sets, 'Choose charset:', default=26)  # Actually 27
        if select < 0:
            exit(-1)
        charset = sets[select].split(' ')[0]

        # Confirm
        if not pause(f'generate ({cyan(min_len)}-{cyan(max_len)}) length dict via "{cyan(charset)}" charset', cancel=True):
            return

        file_name = f'{charset}_{min_len}-{max_len}.dict'
        return self._crunch(lambda: self._gen(file_name, ['crunch', str(min_len), str(max_len), '-f', CHAR_FILE, charset]))



from pathlib import PurePath, Path
from subprocess import check_output
from time import time, strftime, gmtime
from typing import Optional, Callable
from shutil import disk_usage

from .abs_module import Module

from interutils import pr, cyan, choose, pause, human_bytes, count_lines, choose_file, file_volume, IterationTimer


def show_ebt(algos: dict, tlc: int):
    """
    Show estimated burn time
    tlc -> Total lines count
    """
    assert tlc > 0

    for algo, elps in algos.items():
        # Calculate EBT
        ebt = ((tlc * 2) / elps)
        ebt = strftime('%H:%M:%S', gmtime(ebt))
        pr(f'Estimated {cyan(algo)} time: {cyan(ebt)} (assuming elps={elps})')


class Combination(Module):
    def __init__(self, stargen):
        super().__init__(stargen, 'comb')

    def menu(self) -> tuple:
        return {
            'intermix': (self.mix, 'Mix two wordlists'),
            'concat': (self.concat, 'Concatenate two wordlists')
        }

    def ask_two_wl(self, _total_calc=Callable[[int, int], int], _write_action=Callable[[Path, Path, Path, IterationTimer], None]):
        def _select_wordlist(title: str):
            pr(f'Select {title} wordlist:')
            f = choose_file(self.workspace)
            if not f:
                raise KeyboardInterrupt
            fsb, flc, ftxt = file_volume(f)
            pr(f'  {ftxt}')
            return f, fsb, flc

        # Get wordlists
        try:
            f1, f1sb, f1lc = _select_wordlist('first')
            f2, f2sb, f2lc = _select_wordlist('secund')
        except KeyboardInterrupt:
            print()
            return pr('Interrupted', '!')

        # Calculate impact and let the user accept the facts
        tsb = _total_calc(f1sb, f2sb)
        tlc = _total_calc(f1lc, f2lc)
        pr(f'Mixing will allocate {cyan(human_bytes(tsb))} for {cyan("{:,}".format(tlc))} lines')

        free = disk_usage(self.workspace.resolve()).free
        pr(f"Available space on workspace's disk: " + cyan(human_bytes(free)))
        if tsb > free:
            pr('Not enough space on the workspace disk for allocation!', '!')
            return
        max_size = self.config['max_created_file_size']
        if tsb > max_size:
            return pr(f'Calculation resulted in an oversized file (>{human_bytes(max_size)}), aborting!', '!')
        if not pause(cancel=True):
            return

        out_path = self.dest_dir.joinpath(f'{f1.stem}_{f2.stem}')
        with out_path.open('w', encoding='utf-8') as out_file:
            itmr = IterationTimer(tlc, init_interval=1, max_interval=15)
            _write_action(f1, f2, out_file, itmr)

        # Finalize
        pr('Wordlist written into: ' + cyan(out_path.name))
        show_ebt({  # TODO Move to config
            'WPA2': 57000
        }, tlc)

    def mix(self, args: tuple) -> None:
        def mix_action(f1: Path, f2: Path, out_file: Path, itmr: IterationTimer) -> None:
            with f1.open(encoding='utf-8') as f1d:
                for l1 in f1d:
                    if '# ' in l1 or '##' in l1:
                        continue
                    l1 = l1.strip()

                    with f2.open(encoding='utf-8') as f2d:
                        for l2 in f2d:
                            if '# ' in l2 or '##' in l2:
                                continue
                            l2 = l2.strip()

                            out_file.write(f'{l1}{l2}\n{l2}{l1}\n')
                            itmr.tick()  # O(n^2)

        if self.ask_two_wl((lambda a, b: int(a * b * 2)), mix_action) is None:
            return  # Currently redundant - might be of use later

    def concat(self, args: tuple) -> None:
        def concat_action(f1: Path, f2: Path, out_file: Path, itmr: IterationTimer) -> None:
            with f1.open(encoding='utf-8') as f1d:
                for l1 in f1d:
                    if '# ' in l1 or '##' in l1:
                        continue

                    out_file.write(l1)
                    itmr.tick()  # O(n)

            with f2.open(encoding='utf-8') as f2d:
                for l2 in f2d:
                    if '# ' in l2 or '##' in l2:
                        continue

                    out_file.write(l2)
                    itmr.tick()  # O(n)

        if self.ask_two_wl((lambda a, b: a + b), concat_action) is None:
            return  # Currently redundant - might be of use later



from pathlib import Path
from abc import ABCMeta, abstractmethod


class Module(metaclass=ABCMeta):
    def __init__(self, stargen, short_n: str):
        super().__init__()
        # Initialize abstract module configuration
        self.short_n = short_n
        self.stargen = stargen

        self.config = stargen.config['modules'][short_n]
        self.workspace = Path(stargen.config['workspace'])
        self.dest_dir = self.workspace / short_n
        self.dest_dir.mkdir(exist_ok=True)

    @abstractmethod
    def menu(self) -> dict:
        raise NotImplementedError




from pathlib import Path
from argparse import Namespace
from random import choice

from .modules import *

from termcolor import cprint, colored
from interutils import pr, cyan, banner, choose_file, pause, DictConfig


class Stargen:
    DEFAULT_CONFIG_PATH = Path.home().joinpath('.config', 'stargen.json')
    DEFAULT_CONFIG_SETUP = {
        'workspace': str(Path.home().joinpath('dicts').resolve()),
        'prompt': '[ stargen ]> ',
        # Alternatively use "$U" to show filename with extension
        'use_prompt': '[ stargen | $u ]> ',
        'modules': {
            'kwd': {
                'list_treshold': 50
            },
            'down': {
                'dict_url': 'http://ftp.funet.fi/pub/unix/security/passwd/crack/dictionaries/'
            },
            'crun': {
            },
            'comb': {
                'max_created_file_size': 5 * 1024**3,  # 5 GB
                'len_min': 5,
                'len_max': 12,
                'complex_min': 0,
                'complex_max': 100,
            }
        }
    }

    def __init__(self, args: Namespace):
        # Load config
        self.config = DictConfig(
            args.config if args.config else Stargen.DEFAULT_CONFIG_PATH,
            Stargen.DEFAULT_CONFIG_SETUP)

        # Create workspace dir
        self.workspace = Path(self.config['workspace'])
        Path(self.workspace).mkdir(exist_ok=True)

        # Initialize modules
        self.modules = (
            Keyword(self),
            Crunch(self),
            Download(self),
            Combination(self)
        )
        # Initialize menu
        menu = {}
        for mod in self.modules:
            mod: Module
            menu.update(mod.menu())

        # Welcoming message & enter main menu
        cprint(banner('Stargen'), choice(('red', 'green', 'blue')))
        pr(f'Enter "{cyan("help")}" to see list of available commands,\n' +
           '    [Ctrl + C] to exit', '?')
        while 1:
            try:
                # Configure prompt
                current = self.modules[0].current
                if current:
                    prompt = self.config['use_prompt']
                    prompt = prompt.replace('$u', current.stem)
                    prompt = prompt.replace('$U', current.name)
                else:
                    prompt = self.config['prompt']
                # Get user input
                inp = input(colored(prompt, 'red', attrs=['bold']))

                if not inp:
                    continue

                # Help the user
                if 'help' in inp:
                    for k, v in menu.items():
                        print(f'  {cyan(k)} -> {colored(v[1], "yellow")}')
                    continue

                # Get command
                pts = inp.split(' ')
                cmd = pts[0]
                if cmd not in menu:
                    pr(f'No such command! try "{cyan("help")}".', '!')
                    continue

                # Call menu entry
                menu.get(cmd)[0](tuple([i for i in pts[1:] if i]))
                print()
            except (KeyboardInterrupt, EOFError):
                print()
                break
        self.config.save()



from argparse import ArgumentParser, Namespace

from .stargen import Stargen


def parse_args() -> Namespace:
    parser = ArgumentParser('Stargen', description="")

    parser.add_argument('-c', '--config', help='Specify path to config file')

    return parser.parse_args()


def main():
    Stargen(parse_args())


if __name__ == "__main__":
    main()




from urllib.request import urlopen
from pathlib import Path
from os import listdir
import gzip

from .abs_module import Module

from termcolor import cprint
from interutils import pr, cyan

class Download(Module):
    def __init__(self, stargen):
        super().__init__(stargen, 'down')

    def menu(self) -> tuple:
        return {
            # 'show': (self.show, 'Show downloads'),
            'download': (self.download, 'Download a new wordlists pack from the repo')
        }

    def show(self, args: tuple) -> None:

        if not self.packs:
            return pr('No packs downloaded yet!', '!')
        pr('Available packs:')
        for p in self.packs:
            cprint('  ' + p, 'yellow')
        pr(f'Packs count: ' + cyan(len(self.packs)))

    def download(self, args: tuple) -> None:
        # Identify existing downloads
        try:
            self.packs = listdir(self.dest_dir)
        except FileNotFoundError:
            self.packs = []

        pr(f'Dictionaries repo URL: "{cyan(self.config["dict_url"])}"')
        pr(f'Destination directory: "{cyan(str(self.dest_dir))}"')

        print()
        print("	\r\n	Choose the pack you want to download:\r\n")
        print("     1. Moby         14. french     27. places")
        print("     2. afrikaans    15. german     28. polish")
        print("     3. american     16. hindi      29. random")
        print("     4. aussie       17. hungarian  30. religion")
        print("     5. chinese      18. italian    31. russian")
        print("     6. computer     19. japanese   32. science")
        print("     7. croatian     20. latin      33. spanish")
        print("     8. czech        21. literature 34. swahili")
        print("     9. danish       22. movieTV    35. swedish")
        print("    10. databases    23. music      36. turkish")
        print("    11. dictionaries 24. names      37. yiddish")
        print("    12. dutch        25. net")
        print("    13. finnish      26. norwegian       \r\n")

        # Get user input
        try:
            c = input("Enter number > ")
            if not c:
                return pr('Aborting', '!')
            c = int(c)
            if c > 38 or c < 1:
                pr("Bad choice!", '!')
        except ValueError:
            return pr('Bad choice!', '!')

        # List of files to download:
        pack = (
            (
                "Moby",
                (
                    "mhyph.tar.gz",
                    "mlang.tar.gz",
                    "moby.tar.gz",
                    "mpos.tar.gz",
                    "mpron.tar.gz",
                    "mthes.tar.gz",
                    "mwords.tar.gz",
                ),
            ),
            ("afrikaans", ("afr_dbf.zip",)),
            ("american", ("dic-0294.tar.gz",)),
            ("aussie", ("oz.gz",)),
            ("chinese", ("chinese.gz",)),
            (
                "computer",
                (
                    "Domains.gz",
                    "Dosref.gz",
                    "Ftpsites.gz",
                    "Jargon.gz",
                    "common-passwords.txt.gz",
                    "etc-hosts.gz",
                    "foldoc.gz",
                    "language-list.gz",
                    "unix.gz",
                ),
            ),
            ("croatian", ("croatian.gz",)),
            ("czech", ("czech-wordlist-ascii-cstug-novak.gz",)),
            ("danish", ("danish.words.gz", "dansk.zip")),
            (
                "databases",
                ("acronyms.gz", "att800.gz",
                 "computer-companies.gz", "world_heritage.gz"),
            ),
            (
                "dictionaries",
                (
                    "Antworth.gz",
                    "CRL.words.gz",
                    "Roget.words.gz",
                    "Unabr.dict.gz",
                    "Unix.dict.gz",
                    "englex-dict.gz",
                    "knuth_britsh.gz",
                    "knuth_words.gz",
                    "pocket-dic.gz",
                    "shakesp-glossary.gz",
                    "special.eng.gz",
                    "words-english.gz",
                ),
            ),
            ("dutch", ("words.dutch.gz",)),
            (
                "finnish",
                ("finnish.gz", "firstnames.finnish.gz", "words.finnish.FAQ.gz"),
            ),
            ("french", ("dico.gz",)),
            ("german", ("deutsch.dic.gz", "germanl.gz", "words.german.gz")),
            ("hindi", ("hindu-names.gz",)),
            ("hungarian", ("hungarian.gz",)),
            ("italian", ("words.italian.gz",)),
            ("japanese", ("words.japanese.gz",)),
            ("latin", ("wordlist.aug.gz",)),
            (
                "literature",
                (
                    "LCarrol.gz",
                    "Paradise.Lost.gz",
                    "aeneid.gz",
                    "arthur.gz",
                    "cartoon.gz",
                    "cartoons-olivier.gz",
                    "charlemagne.gz",
                    "fable.gz",
                    "iliad.gz",
                    "myths-legends.gz",
                    "odyssey.gz",
                    "sf.gz",
                    "shakespeare.gz",
                    "tolkien.words.gz",
                ),
            ),
            ("movieTV", ("Movies.gz", "Python.gz", "Trek.gz")),
            (
                "music",
                (
                    "music-classical.gz",
                    "music-country.gz",
                    "music-jazz.gz",
                    "music-other.gz",
                    "music-rock.gz",
                    "music-shows.gz",
                    "rock-groups.gz",
                ),
            ),
            (
                "names",
                (
                    "ASSurnames.gz",
                    "Congress.gz",
                    "Family-Names.gz",
                    "Given-Names.gz",
                    "actor-givenname.gz",
                    "actor-surname.gz",
                    "cis-givenname.gz",
                    "cis-surname.gz",
                    "crl-names.gz",
                    "famous.gz",
                    "fast-names.gz",
                    "female-names-kantr.gz",
                    "female-names.gz",
                    "givennames-ol.gz",
                    "male-names-kantr.gz",
                    "male-names.gz",
                    "movie-characters.gz",
                    "names.french.gz",
                    "names.hp.gz",
                    "other-names.gz",
                    "shakesp-names.gz",
                    "surnames-ol.gz",
                    "surnames.finnish.gz",
                    "usenet-names.gz",
                ),
            ),
            (
                "net",
                (
                    "hosts-txt.gz",
                    "inet-machines.gz",
                    "usenet-loginids.gz",
                    "usenet-machines.gz",
                    "uunet-sites.gz",
                ),
            ),
            ("norwegian", ("words.norwegian.gz",)),
            (
                "places",
                (
                    "Colleges.gz",
                    "US-counties.gz",
                    "World.factbook.gz",
                    "Zipcodes.gz",
                    "places.gz",
                ),
            ),
            ("polish", ("words.polish.gz",)),
            (
                "random",
                (
                    "Ethnologue.gz",
                    "abbr.gz",
                    "chars.gz",
                    "dogs.gz",
                    "drugs.gz",
                    "junk.gz",
                    "numbers.gz",
                    "phrases.gz",
                    "sports.gz",
                    "statistics.gz",
                ),
            ),
            ("religion", ("Koran.gz", "kjbible.gz", "norse.gz")),
            ("russian", ("russian.lst.gz", "russian_words.koi8.gz")),
            (
                "science",
                (
                    "Acr-diagnosis.gz",
                    "Algae.gz",
                    "Bacteria.gz",
                    "Fungi.gz",
                    "Microalgae.gz",
                    "Viruses.gz",
                    "asteroids.gz",
                    "biology.gz",
                    "tech.gz",
                ),
            ),
            ("spanish", ("words.spanish.gz",)),
            ("swahili", ("swahili.gz",)),
            ("swedish", ("words.swedish.gz",)),
            ("turkish", ("turkish.dict.gz",)),
            ("yiddish", ("yiddish.gz",)),
        )[c - 1]

        if pack in self.packs:
            return pr('Pack already downloaded!', '*')

        # Verify destination directory
        dest = Path(self.dest_dir)
        dest.mkdir(exist_ok=True)
        dest = dest / pack[0]
        dest.mkdir(exist_ok=True)

        # Download the files
        for fi in pack[1]:
            pf = pack[0] + "/" + fi
            pr(f"Downloading & ungzipping '{cyan(pf)}'")
            with urlopen(self.config['dict_url'] + pf) as web:
                dest.joinpath(fi.replace('.gz', '')).write_bytes(
                    gzip.decompress(web.read()))

        # Finish
        self.packs.append(pack[0])
        pr("Pack saved to " + str(dest))
        pr("Tip: After downloading wordlists combine them with keywords!", '*')



from pathlib import Path
from typing import Callable
from time import time
from shutil import copy, move
from tempfile import mkstemp

from .abs_module import Module

from termcolor import cprint, colored
from interutils import pr, cyan, pause, choose_file, file_volume, IterationTimer, count_lines, ask


def mockify(text: str, start_first: bool) -> str:
    r = ''
    for i in range(len(text)):
        c = text[i]
        if i % 2 == int(start_first):
            c = c.swapcase()
        r += c
    return r


def leetify(text: str):
    result = text.translate(str.maketrans('AaSsIiEeOoTtBb', '@@551133007788'))
    return result


class Keyword(Module):
    def __init__(self, stargen):
        super().__init__(stargen, 'kwd')
        self.current: Path = None

    def _modifier_wrapper(self, tmp_path: Path, name: str, impact: str, ask: bool, modifier: Callable[[int], None]) -> None:
        count = count_lines(tmp_path)
        if ask and not pause(f'{cyan(name)} keywords (impact: {impact} => {eval(f"{count}{impact}")})', cancel=True):
            return
        modifier(tmp_path, count)
        new_count = count_lines(tmp_path)
        pr(f'{name} added {cyan(new_count - count)} new keywords, total: {cyan(new_count)}')

    def _get_wordlist_path(self):
        if self.current:
            return self.current
        return choose_file(self.workspace)

    def _gen_wordlist(self, wordlist: Path):
        if wordlist:
            f = wordlist
        else:
            f = self._get_wordlist_path()
            if not f:
                return
        for l in f.read_text(encoding='utf8').splitlines():
            yield l

    def menu(self) -> tuple:
        return {
            'use': (self.use, 'Specify which wordlist to use for operations'),
            'show': (self.print_all, 'List all keywords and show count\n\tOpt: "total" -> Print sum total'),
            'expand': (self.expand, 'Expand keywords\n\tOpt: "all" -> Execute all modifications'),
            'add': (self.add, 'Add keyword(s)'),
            # 'rem': (self.rem, 'Remove keyword(s)'),
            'clear': (self.clear, 'Clear all keywords'),
            'duplicate': (self.duplicate, 'Duplicate the wordlist\n\tReq: "name" -> Duplicated wordlist file name'),
            'isin': (self.isin, 'Check if string(s) among keywords')
        }

    def use(self, args: tuple) -> None:
        if self.current:
            if not pause(f'Switching from {cyan(self.current)} to new wordlist', cancel=True):
                return
        f = choose_file(self.workspace)
        if not f:
            return

        sb, lc, txt = file_volume(f)
        pr(txt)

        # if sb > 1*1024**3:  # 1 GB
        #     pr(f'File is too larget to be loaded into RAM!', '!')
        #     return
        self.current = f
        pr(f'Now using {cyan(self.current)} wordlist!')

    def print_all(self, args: tuple) -> None:
        f = self._get_wordlist_path()
        if not f:
            return

        lc = count_lines(f)
        if not lc:
            return pr('Wordlist is empty!', '!')

        # Get arguments
        total = False
        if args:
            total = args[0] == 'total'

        # Print relevant info
        if not total:
            if lc > self.config['list_treshold']:
                if not pause(f'show all {cyan(lc)} keywords', cancel=True):
                    return
            for v in self._gen_wordlist(f):
                cprint('  ' + v, "yellow")
        pr(f'Total keywords count: ' + cyan(lc))

    def expand(self, args: tuple) -> None:
        f = self._get_wordlist_path()
        if not f:
            return

        lc = count_lines(f)
        if not self:
            return pr('Wordlist is empty!', '!')

        # Get new wordlist name
        try:
            print('Enter name for new expanded wordlist:')
            save_path = input(colored('>', 'yellow'))
            if not save_path:
                return
            save_path = self.workspace.joinpath('expand_' + str(save_path))
            if save_path.is_file():
                pr(f'File {cyan(save_path)} already exists, overwrite?', '!')
                if not pause(cancel=True):
                    return
        except KeyboardInterrupt:
            return

        # Get arguments
        auto_all = False
        if args:
            auto_all = args[0] == 'all'

        _, tmp_path = mkstemp('stargen')
        tmp_path = Path(tmp_path)

        # Copy initial content
        tmp_path.write_bytes(f.read_bytes())

        def _capitalize(tmp_p: Path, count: int) -> None:
            with tmp_p.open('a', encoding='utf8') as file:
                for k in self._gen_wordlist(f):
                    file.write(k.capitalize() + '\n')
        self._modifier_wrapper(tmp_path, 'Capitalize', '*2',
                               not auto_all, _capitalize)

        def _leetify(tmp_p: Path, count: int) -> None:
            with tmp_p.open('a', encoding='utf8') as file:
                for k in self._gen_wordlist(f):
                    file.write(leetify(k) + '\n')
        self._modifier_wrapper(tmp_path, '13371fy', '*2',
                               not auto_all, _leetify)

        def _mockify(tmp_p: Path, count: int) -> None:
            with tmp_p.open('a', encoding='utf8') as file:
                for k in self._gen_wordlist(f):
                    file.write(mockify(k, True) + '\n')
                    file.write(mockify(k, False) + '\n')
        self._modifier_wrapper(tmp_path, 'MoCkIfY', '*3',
                               not auto_all, _mockify)

        def _intermix(tmp_p: Path, count: int) -> None:
            itmr = IterationTimer(count ** 2)
            with tmp_p.open('a', encoding='utf8') as file:
                for a in self._gen_wordlist(f):
                    for b in self._gen_wordlist(f):
                        file.write(a + b + '\n')
                        file.write(b + a + '\n')
                        itmr.tick()
        self._modifier_wrapper(tmp_path, 'Intermix', '**2',
                               not auto_all, _intermix)

        # Save as
        pr(f'Saving as: {cyan(save_path)}')
        move(tmp_path, save_path)
        new_lc = count_lines(save_path)

        # Show current status
        print()
        a = []
        if new_lc > self.config['list_treshold']:
            a += ['total']
        self.print_all(a)

    def add(self, args: tuple) -> None:
        if not args:
            return pr('Usage: add <keyword...>', '*')

        f = self._get_wordlist_path()
        if not f:
            return

        for a in args:
            if a in self._gen_wordlist(f):
                pr(f'Skipping duplicate "{cyan(a)}"', '*')
                continue
            pr(f'Adding "{a}"')
            with f.open('a') as file:
                file.write(a)

    def clear(self, args: tuple) -> None:
        if not pause('truncatete wordlist', cancel=True):
            return

        f = self._get_wordlist_path()
        if not f:
            return

        with f.open('w') as file:
            file.truncate()
        pr('Wordlist truncateted!')

    def duplicate(self, args: tuple) -> None:
        if not args:
            return pr('Usage: duplicate <copy_name>', '*')

        f = self._get_wordlist_path()
        if not f:
            return

        dst = Path(args[0])
        if dst.is_file():
            pr(f'File {cyan(dst)} already exists, overwrite?', '!')
            if not pause(cancel=True):
                return
        copy(f, dst)
        pr(f'Copied to {cyan(dst)}!')

    def isin(self, args: tuple) -> None:
        if not args:
            return pr('Usage: isin <keyword...>', '*')

        f = self._get_wordlist_path()
        if not f:
            return

        for a in args:
            if a in self._gen_wordlist(f):
                pr(f'Found a match for {cyan(a)}')
            else:
                pr(f'No match for {cyan(a)}', '!')



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="stargen",
    version="0.8.4",
    description="Interactive wordlists generator",
    url="https://github.com/codeswhite/stargen",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
        'interutils',
        'pyfiglet',
    ],
    entry_points={
        'console_scripts': [
            'stargen = stargen:main',
        ],
    },
    author="Max G",
    author_email="max3227@gmail.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages()
)
